;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                 M5481-Routine               ;;
;;                 =============               ;;
;;             (c)acer_net@yahoo.de            ;;
;;           Version 1, September 2008		   ;;
;;                                             ;;
;; Zwei 7-Segment-Anzeigen ansteuern mit dem   ;;
;; LED-Display-Treiber M5481 von ST            ;;
;;											   ;;	
;; 3-Bit-Ansteuerung an eimen ATMega           ;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Hardware: M5481 am aktiven Port
;
; Definitionen, die in dem aufrufenden Assembler-
; Programm enthalten sein mssen:
; - Stackoperationen
; - Temp1 (R16-R29)     Zahl 0-99
; - LEDPort				Aktiver M5481-Port
; - LEDDdr				Datenrichtungsregister des aktiven Port (alle 3 Pin auf Ausgang)
; - LED_Clock  			Portpin fr Clock   (11)
; - LED_Data 			Portpin fr Data in (12)
; - LED_EN				Portpin fr /Enable (13)

.DEF Zero	= R1
.DEF LTemp2 = R17
.DEF LTemp3 = R18
.DEF L10	= R19
.DEF L1		= R20


; Unterprogramm wird mit rcall  LED_OUT aufgerufen
; Die Zahl mu zuvor in temp1 stehen
; Es werden keine Register verndert!

LED_OUT:
		push  ZL
		push  ZH
		push  R0						; Register retten
		push  R1
		push  R16
		push  R17
    	push  R18
		push  R19
		push  R20
		
		;****** Zahl in Einer und Zehner zerlegen ******	
		ldi   LTemp2, 0
		mov   Zero,LTemp2

Ten:
        subi  Temp1, 10					; abzhlen wieviele Zehner in
        brcs  TenOk						; der Zahl enthalten sind
        inc   LTemp2
        rjmp  Ten

                                    
TenOk:									; die Zehnerstelle berechnen
        ldi   ZL, LOW(Codes*2)			; die Startadresse der Tabelle in den
        ldi   ZH, HIGH(Codes*2)			; Z-Pointer laden
		
		mov   LTemp3, LTemp2			; die wortweise Adressierung der Tabelle
        add   LTemp3, LTemp2		    ; bercksichtigen
 
        add   ZL, LTemp3		        ; und ausgehend vom Tabellenanfang
        adc   ZH, zero                	; die Adresse des Code Bytes berechnen

		lpm
		mov   L10,r0

		subi  Temp1, -10	            ; 10 wieder dazuzhlen, da die
                                        ; vorhergehende Schleife 10 zuviel
                                        ; abgezogen hat
	                                    ; die brig gebliebenen Einer
                                        ; noch ausgeben
        ldi   LTemp2, 0
        add   Temp1, LTemp2

		ldi   ZL, LOW(Codes*2)			; die Startadresse der Tabelle in den
        ldi   ZH, HIGH(Codes*2)			; Z-Pointer laden
		
		mov   LTemp3, Temp1				; die wortweise Adressierung der Tabelle
        add   LTemp3, Temp1			    ; bercksichtigen
 
        add   ZL, LTemp3		        ; und ausgehend vom Tabellenanfang
        adc   ZH, zero                	; die Adresse des Code Bytes berechnen

		lpm
		mov   L1,r0


		;******* Die 35 Code-Bits zusammenstellen und bertragen *******
		ldi   LTemp2,35					; Anzahl zu bertragener Bits 
		rcall LED_ST					; bertragung einleiten

		mov   temp1,L1					; Teil 1 von Einer-Byte bertragen (Bit 4-7 sind relevant)
		rcall LD3 

		swap  L1						; Einer-Byte richtigstellen 
		rol   L1						; Highest-Bit verwerfen
		swap  L10						; Zehner-Byte richtigstellen
		ror	  L10						; Lower-Bit in Carry
		ror	  L1						; Carry in Zehner-Byte schieben
		
		mov   temp1,L1					; Teil 2 von Einer-Byte bertragen (Bit 4-7 sind relevant)
		rcall LD3

		swap  L10						; Zehner-Byte umdrehen
	
		mov   temp1,L10					; Teil 1 von Zehner-Byte bertragen (Bit 4-7 sind relevant)
		rcall LD3
	
		swap  L10						; Zehner-Byte wieder drehen und

		mov   temp1,L10					; Teil 2 von Zehner-Byte bertragen (Bit 4-7 sind relevant)
		rcall LD3
		rcall LD3						; Restliche unrelevante Bits bertagen bis Anzahl von 35
										; erreicht ist
		rjmp  LDENDE
		
		
		;****** Start bertragung ********			
LED_ST:	cbi   LEDPort,LED_EN			; Senden enablen
		sbi   LEDPort,LED_Data			; 1. Bit immer 1
		rcall LedCL1					; Clocken
		ret
		
		;****** Byte bitweise bertragen *********
LD3:	ldi   LTemp3,8					; 8 Bit		
LD2:	cbi   LEDPort,LED_Data			; Data auf Low
		ror   Temp1						; Bit in Carry schieben	
		brcc  LD1						; Wenn Carry gesetzt weiter, sonst
		sbi   LEDPort,LED_Data			; Data auf High
		nop
LD1:	rcall LedCL1					; Clock-Impuls ausgeben
		dec   LTemp2					; 35 Bits bertragen?
		breq  EE						; Ja
		dec	  LTemp3					; Nein, Bitzhler decrementieren
		brne  LD2						; 8 Bits bertragen

EE:		ret
	
		;***** bertragung beenden und Register wieder herstellen *******
LDEnde:	sbi   LEDPort,LED_EN

		pop   R20
		pop   R19
		pop   R18
		pop   R17
		pop   R16
		pop   R1
		pop   R0
		pop   ZH
		pop   ZL
		
		ret								;Fertig

		;***** Clock-Impuls erzeugen ********
LedCL1: sbi   LEDPort,LED_Clock
		nop
		nop
		cbi   LEDPort,LED_Clock
		ret

		;***** LED-Code-Tabelle **********************
Codes:									; Die Codetabelle fr die Ziffern 0 bis 9
                                     	; sie regelt, welche Segmente fr eine
                                     	; Ziffer eingeschaltet werden mssen
										; Unteres und oberes Nibble werden getrennt behandelt
                                     	;
		.db    0b11110011       		; 0: d, c, b, a, f, e
		.db    0b01100000        		; 1: c, b
        .db    0b10110101        		; 2  d, b, a, g, e
        .db    0b11110100        		; 3: d, c, b, a, g
        .db    0b01100110        		; 4: c, b, g, f
        .db    0b11010110        		; 5: d, c, a, g, f
        .db    0b11010111        		; 6: d, c, a, g, f, e
        .db    0b01110000        		; 7: c, b, a
        .db    0b11110111        		; 8: d, c, b, a, g, f, e
        .db    0b11110110        		; 9: d, c, b, a, g, f
